#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-12-23
# File  : bgp_topology/lib/args.py


from argparse import ArgumentParser, Namespace, RawTextHelpFormatter
from collections.abc import Sequence

from cmk_addons.plugins.bgp_topology.constants import (
    __AUTHOR__,
    __URL__,
    __USAGE__,
    __VERSION__,
    ARG_PARSER_ANCHOR,
    ARG_PARSER_ANCHOR_AS,
    ARG_PARSER_ANCHOR_BOTH,
    ARG_PARSER_ANCHOR_ID,
    ARG_PARSER_EMBLEM_AS,
    ARG_PARSER_EMBLEM_ID,
    ARG_PARSER_HOST,
    ARG_PARSER_MAKE_DEFAULT,
    ARG_PARSER_NONE,
    ARG_PARSER_SITES_EXCLUDE,
    ARG_PARSER_SITES_INCLUDE,
    ARG_PARSER_DEBUG,
    EMBLEM_BGP_AS,
    EMBLEM_BGP_ID,
)
from cmk_addons.plugins.bgp_topology.lib.utils import (
    OMD_ROOT,
)


class Params(Namespace):
    bgp_anchor: str
    make_default: bool = False
    bgp_emblem_as: str = EMBLEM_BGP_AS
    bgp_emblem_id: str = EMBLEM_BGP_ID
    include_sites: Sequence[str] | None = None
    exclude_sites: Sequence[str] | None = None
    host: str = ''
    debug: bool = False


def parse_arguments(argv: Sequence[str]) -> Params:
    parser = ArgumentParser(
        prog='bgp_topology',
        formatter_class=RawTextHelpFormatter,
        description=f"""Create BGP peer network topology for Checkmk""",
        epilog=f"""
Example usage:
{__USAGE__}        

Version: {__VERSION__} | Written by {__AUTHOR__}
for more information see: {__URL__}
        """
    )
    parser.add_argument(
        ARG_PARSER_ANCHOR,
        choices=[
            ARG_PARSER_ANCHOR_BOTH,
            ARG_PARSER_ANCHOR_AS,
            ARG_PARSER_ANCHOR_ID,
            ARG_PARSER_NONE,
        ],
        default=ARG_PARSER_ANCHOR_BOTH,
        help='Anchor for external BGP objects (default: %(default)s).',
    )
    parser.add_argument(
        ARG_PARSER_EMBLEM_AS,
        type=str,
        default=EMBLEM_BGP_AS,
        help='Emblem to use for BGP-AS objects (default: %(default)s).',
    )
    parser.add_argument(
        ARG_PARSER_EMBLEM_ID,
        type=str,
        default=EMBLEM_BGP_ID,
        help='Emblem to use for BGP-ID objects (default: %(default)s).',
    )
    parser.add_argument(
        ARG_PARSER_MAKE_DEFAULT,
        action='store_const', const=True,
        default=False,
        help='Make this topology the default (default: %(default)s).',
    )
    parser.add_argument(
        ARG_PARSER_HOST,
        required=True,
        type=str,
        help="""The name of the Checkmk host to which the plugin is attached. This is set 
automatically by Checkmk. If a site filter is active, the host name is 
appended to the subdirectory where the topology is stored (“BGP” becomes 
“BGP_host_name”). This way we can have more than one BGP topology without 
overwriting each other.""",
    )
    site_filter = parser.add_mutually_exclusive_group()
    site_filter.add_argument(
        ARG_PARSER_SITES_INCLUDE,
        type=str,
        nargs='+',
        help=f"""List of Checkmk site names to include in the topology creation. 
Can not used together with {ARG_PARSER_SITES_EXCLUDE}""",
    )
    site_filter.add_argument(
        ARG_PARSER_SITES_EXCLUDE,
        type=str,
        nargs='+',
        help=f"""List of Checkmk site names to exclude from the topology creation. 
Can not used together with {ARG_PARSER_SITES_INCLUDE}""",
    )
    parser.add_argument(
        ARG_PARSER_DEBUG,
        action='store_const', const=True,
        default=False,
        help='enable debug output',
    )

    return parser.parse_args(argv)

