#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-07-16
# File  : bgp_topology/lib/utils.py

# 2024-07-16: copied from vsphere_topology/lib/utils.py

from collections.abc import Mapping, MutableMapping, MutableSequence, Sequence
from dataclasses import dataclass
from json import dumps as json_dunps, loads as json_loads
from os import environ
from pathlib import Path
from typing import Final, Tuple

from livestatus import MultiSiteConnection, SiteConfigurations, SiteId
from cmk_addons.plugins.bgp_topology.constants import (
    ARG_PARSER_NONE,
    BGP_PEER_LOCAL_ADDR,
    BGP_PEER_LOCAL_AS,
    BGP_PEER_LOCAL_ID,
    BGP_PEER_REMOTE_ADDR,
    BGP_PEER_REMOTE_AS,
    BGP_PEER_REMOTE_ID,
    BGP_PEER_STATE,
    BGP_PEER_UPTIME,
    PARAM_BGP_EXT_ANCHOR_AS,
    PARAM_BGP_EXT_ANCHOR_BOTH,
    PARAM_BGP_EXT_ANCHOR_ID,
    TOPOLOGY_NAME,
)

OMD_ROOT = environ['OMD_ROOT']
BASE_TOPO_PATH: Final[str] = f'{OMD_ROOT}/var/check_mk/topology/data'


def get_bgp_peer_clean_key(raw_key: str) -> str | None:
    key_map: Mapping = {
        'Local AS': BGP_PEER_LOCAL_AS,
        'Local address': BGP_PEER_LOCAL_ADDR,
        'Local identifier': BGP_PEER_LOCAL_ID,
        'Peer state': BGP_PEER_STATE,
        'Remote AS': BGP_PEER_REMOTE_AS,
        'Remote address': BGP_PEER_REMOTE_ADDR,
        'Remote identifier': BGP_PEER_REMOTE_ID,
        'Uptime': BGP_PEER_UPTIME,
    }

    return key_map.get(raw_key)


def get_bgp_peer_clean_value(key: str, value: str) -> str | None:
    class MatchValues:
        local_addr = BGP_PEER_LOCAL_ADDR
        remote_addr = BGP_PEER_REMOTE_ADDR
        remote_id = BGP_PEER_REMOTE_ID

    if value is not None:
        value = value.strip()
    match key:
        case MatchValues.local_addr | MatchValues.remote_addr | MatchValues.remote_id:
            if value.strip() in ['0.0.0.0', 'N/A']:
                return
    if value:
        return value


def get_service(host: str, servie: str) -> str:
    return f'{servie}@{host}'


def get_anchor(raw_anchor: str) -> str | None:
    anchor_map = {
        'both': PARAM_BGP_EXT_ANCHOR_BOTH,
        'as': PARAM_BGP_EXT_ANCHOR_AS,
        'id': PARAM_BGP_EXT_ANCHOR_ID,
        'none': ARG_PARSER_NONE
    }
    return anchor_map.get(raw_anchor)


def get_emblem(emblem: str) -> str | None:
    if emblem != ARG_PARSER_NONE:
        return emblem


def save_topology(data: Mapping, sub_directory: str) -> None:
    """
    Save the topology as json file under $OMD_ROOT/var/check_mk/topology/data/{sub_directory}.data_{data['name']}.json
    the filename will be changed to lower case.
    Args:
        data: the topology data
        sub_directory: the subdirectory were to save the data under ~/var/check_mk/topology/data/

    Returns:
        None
    """
    file_name = f'data_{data["name"]}.json'.lower()
    save_file = Path(f'{BASE_TOPO_PATH}/{sub_directory}/{file_name}')
    save_file.parent.mkdir(exist_ok=True, parents=True)
    save_file.write_text(json_dunps(data))


def make_topo_default(sub_directory: str, make_default: bool) -> None:
    """
    Create the symlink "default" to $OMD_ROOT/var/check_mk/topology/data/{sub_directory} in
    $OMD_ROOT/var/check_mk/topology/data/ if it don't exist or mage_default is True
    Args:
        sub_directory: the subdirectory under ~/var/check_mk/topology/data/ thaht become default
        make_default: if True, create the symlink "default" with path as target

    Returns:
        None
    """

    target_path = f'{BASE_TOPO_PATH}/{sub_directory}'

    if not Path(f'{BASE_TOPO_PATH}/default').exists():
        make_default = True
    if make_default:
        Path(f'{BASE_TOPO_PATH}/default').unlink(missing_ok=True)
        Path(f'{BASE_TOPO_PATH}/default').symlink_to(target=Path(target_path), target_is_directory=True)


def get_topologies() -> Sequence[str | None]:
    """
    Returns a list of topology names form the default typology directory.

    Returns:
        List of str ie: ['CDP', 'LLDP']
    """
    path: str = f'{BASE_TOPO_PATH}/default'
    if not Path(path).exists():
        return []

    files = [f for f in Path(path).glob('*.json') if f.is_file()]
    return [
        json_loads(Path(file).read_text())['name'] for file in files if
        json_loads(Path(file).read_text()).get('name') is not None
    ]


def add_dummy_topologies(sub_directory: str):
    path: str = f'{BASE_TOPO_PATH}/default'

    # don't overwrite existing topology
    if Path(path).exists() and not Path(f'{path}/data_{TOPOLOGY_NAME.lower()}.json').exists():
        dummy_topology = {'version': 1, 'name': TOPOLOGY_NAME, 'objects': {}, 'connections': []}
        save_topology(
            data=dummy_topology,
            sub_directory='default',
        )

    for topology in get_topologies():
        if not Path(f'{BASE_TOPO_PATH}/{sub_directory}/data_{topology.lower()}.json').exists():
            save_topology(
                data={'version': 1, 'name': topology, 'objects': {}, 'connections': []},
                sub_directory=sub_directory
            )


#
#  live status
#
class LiveStatusConnection:
    def __init__(self):
        self.sites: SiteConfigurations = SiteConfigurations({})
        self.sites_mk = Path(f'{OMD_ROOT}/etc/check_mk/multisite.d/sites.mk')
        self.socket_path = f'unix:{OMD_ROOT}/tmp/run/live'
        if self.sites_mk.exists():
            # make eval() "secure"
            # https://realpython.com/python-eval-function/#minimizing-the-security-issues-of-eval
            _code = compile(self.sites_mk.read_text(), '<string>', 'eval')
            allowed_names = ['sites', 'update']
            for name in _code.co_names:
                if name not in allowed_names:
                    raise NameError(f'Use of {name} in {self.sites_mk.name} not allowed.')

            sites_raw: MutableMapping = {}
            eval(self.sites_mk.read_text(), {'__builtins__': {}}, {'sites': sites_raw})

            for site, data in sites_raw.items():
                self.sites.update({site: {
                    'alias': data['alias'],
                    'timeout': data['timeout'],
                }})
                if data['socket'] == ('local', None):
                    self.sites[site]['socket'] = self.socket_path
                else:
                    protocol, socket = data['socket']
                    address, port = socket['address']
                    self.sites[site]['socket'] = f'{protocol}:{address}:{port}'
                    self.sites[site]['tls'] = socket['tls']
        else:
            self.sites.update({SiteId('local'): {
                'alias': 'Local site',
                'timeout': 5,
                'socket': self.socket_path
            }})

        self.c = MultiSiteConnection(self.sites)
        dead_sites = [site['site']['alias'] for site in self.c.dead_sites().values()]
        if dead_sites:
            self.c.set_only_sites(self.c.alive_sites())

    def query(self, query: str):
        data: MutableSequence[Tuple[str, str]] = self.c.query(query=query)

        if data:
            return data
        else:
            return None

    def filter_sites(self, include: bool, sites: Sequence[str]):
        if include is True:
            site_list = [site for site in self.c.sites if site in sites]
        else:
            site_list = [site for site in self.c.sites if site not in sites]

        self.c.set_only_sites(site_list)


class TopoObjects:
    def __init__(self) -> None:
        self.topo_objects: MutableMapping[str, object] = {}

    def add_host(
            self,
            host: str,
            emblem: str | None = None,
            icon: str | None = None,
            link2core: bool = True,
            obj_id_prefix: str = '',
    ):
        if self.topo_objects.get(f'{obj_id_prefix}{host}') is not None:
            return

        metadata = {}
        if emblem or icon:
            metadata = {'images': {}}

        if emblem is not None:
            metadata['images'].update({'emblem': emblem})  # node image

        if icon is not None:
            metadata['images'].update({'icon': icon})  # node icon

        self.topo_objects[f'{obj_id_prefix}{host}'] = {
            'name': host,
            'link': {'core': host} if link2core else {},
            'metadata': metadata,
        }

    def add_service(
            self,
            host: str,
            service: str,
            emblem: str | None = None,
            link2core: bool = True,
    ):
        obj_id = f'{service}@{host}'
        if self.topo_objects.get(obj_id) is not None:
            return

        metadata = {}
        if emblem is not None:
            metadata = {
                'images': {
                    'emblem': emblem,  # node image
                },
            }

        self.topo_objects[obj_id] = {
            'name': service,
            'link': {'core': [host, service]} if link2core else {},
            'metadata': metadata,
        }


def get_connection_color(left_state: int, right_state:int) -> str:
    state_to_color = {
        0: 'white',
        1: 'yellow',
        2: 'red',
        3: 'orange',
    }

    return state_to_color.get(max(left_state, right_state), 'orange')


class TopoConnections:
    def __init__(self) -> None:
        self.topo_connections: MutableSequence = []
        self.clean_connections: MutableSequence[Sequence[str]] = []

    def add_connection(
            self,
            left: str,
            right: str,
            left_state: int = 0,
            right_state: int = 0,
    ):
        connection = [left, right]
        connection.sort()
        if connection not in self.clean_connections:
            self.clean_connections.append(connection)
            self.topo_connections.append([
                connection, {
                    'line_config': {
                        'css_styles': {
                            'stroke-dasharray': 'unset'
                        },
                        'color': get_connection_color(left_state, right_state),
                    }
                }
            ])


# taken from https://github.com/Checkmk/checkmk/blob/master/cmk/plugins/smb/lib/check_disk_smb.py
# Change-Id: I7426f6553a906c5ac50a8306157931a10640a526
@dataclass
class Metric:
    name: str
    value: float
    levels: tuple[float, float] | None = None
    boundaries: tuple[float, float] | None = None

    def __str__(self) -> str:
        l = f"{self.levels[0]};{self.levels[1]};" if self.levels else ";;"
        b = f"{self.boundaries[0]};{self.boundaries[1]}" if self.boundaries else ";"
        # I'm not too sure about the single quotes here, but keeping it for now
        return f"'{self.name}'={self.value}B;{l}{b}"