#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-07-06
# File  : bgp_topology/rulesets/bgp_topology.py

from collections.abc import Sequence

from cmk.rulesets.v1 import Help, Label, Message, Title
from cmk.rulesets.v1.form_specs import (
    CascadingSingleChoice,
    CascadingSingleChoiceElement,
    DefaultValue,
    DictElement,
    Dictionary,
    FixedValue,
    SingleChoice,
    SingleChoiceElement,
    String,
    List
)

from cmk.rulesets.v1.form_specs.validators import LengthInRange, ValidationError
from cmk.rulesets.v1.rule_specs import ActiveCheck, Topic
from cmk_addons.plugins.bgp_topology.constants import (
    EMBLEM_BGP_AS,
    EMBLEM_BGP_ID,
    PARAM_BGP_AS,
    PARAM_BGP_EXT_ANCHOR,
    PARAM_BGP_EXT_ANCHOR_AS,
    PARAM_BGP_EXT_ANCHOR_BOTH,
    PARAM_BGP_EXT_ANCHOR_ID,
    PARAM_BGP_EXT_ANCHOR_NONE,
    PARAM_BGP_ID,
    PARAM_BGP_SITES_EXCLUDE,
    PARAM_BGP_SITES_FILTER,
    PARAM_BGP_SITES_INCLUDE,
    PARAM_EMBLEM_CUSTOM,
    PARAM_EMBLEM_DEFAULT,
    PARAM_EMBLEM_NO_EMBLEM,
    PARAM_MAKE_DEFAULT,
    PICTURE_TYPE_EMBLEM,
    RULE_SET_NAME_BGP_TOPOLOGY,
)


class DuplicateInList:  # pylint: disable=too-few-public-methods
    """ Custom validator that ensures the validated list has no duplicate entries. """

    def __init__(
            self,
    ) -> None:
        pass

    @staticmethod
    def _get_default_errmsg(_duplicates: Sequence) -> Message:
        return Message(f"Duplicate element in list. Duplicate elements: {', '.join(_duplicates)}")

    def __call__(self, value: List[str] | None) -> None:
        if not isinstance(value, list):
            return
        _duplicates = [value[i] for i, x in enumerate(value) if value.count(x) > 1]
        _duplicates = list(set(_duplicates))
        if _duplicates:
            raise ValidationError(message=self._get_default_errmsg(_duplicates))


def get_emblem_element(default_emblem: str, picture_type: str) -> Sequence[CascadingSingleChoiceElement]:
    return [
        CascadingSingleChoiceElement(
            name=PARAM_EMBLEM_NO_EMBLEM,
            title=Title(f'No custom {picture_type}'),
            parameter_form=FixedValue(
                value=True,
                label=Label(f'No custom {picture_type} will be used')
            )),
        CascadingSingleChoiceElement(
            name=PARAM_EMBLEM_DEFAULT,
            title=Title(f'Use default {picture_type}'),
            parameter_form=FixedValue(
                value=True,
                label=Label(f'"{default_emblem}" will be used as {picture_type}')
            )),
        CascadingSingleChoiceElement(
            name=PARAM_EMBLEM_CUSTOM,
            title=Title(f'Use custom {picture_type}'),
            parameter_form=String(
                custom_validate=(LengthInRange(min_value=1),),
                prefill=DefaultValue(default_emblem),
            ))
    ]


def _parameter_form() -> Dictionary:
    return Dictionary(
        elements={
            PARAM_BGP_AS: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('BGP AS emblem'),
                    elements=get_emblem_element(EMBLEM_BGP_AS, PICTURE_TYPE_EMBLEM),
                    prefill=DefaultValue(PARAM_EMBLEM_DEFAULT),
                    help_text=Help(
                        'Here you can change the picture for the BGP-AS object. '
                        'If you use the built-in icons prefix the name with "icon_"'
                    ),
                )),
            PARAM_BGP_ID: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('BGP ID emblem'),
                    elements=get_emblem_element(EMBLEM_BGP_ID, PICTURE_TYPE_EMBLEM),
                    prefill=DefaultValue(PARAM_EMBLEM_DEFAULT),
                    help_text=Help(
                        'Here you can change the picture for the BGP-ID object. '
                        'If you use the built-in icons prefix the name with "icon_"'
                    ),
                )),
            PARAM_BGP_EXT_ANCHOR: DictElement(
                parameter_form=SingleChoice(
                    title=Title('Anchor for external BGP objects'),
                    elements=[
                        SingleChoiceElement(name=PARAM_BGP_EXT_ANCHOR_BOTH, title=Title('Use BGP AS and ID')),
                        SingleChoiceElement(name=PARAM_BGP_EXT_ANCHOR_AS, title=Title('Use BGP AS only')),
                        SingleChoiceElement(name=PARAM_BGP_EXT_ANCHOR_ID, title=Title('Use BGP ID only')),
                        SingleChoiceElement(name=PARAM_BGP_EXT_ANCHOR_NONE, title=Title(f'No anchor object'))
                    ],
                    prefill=DefaultValue(PARAM_BGP_EXT_ANCHOR_BOTH),
                    help_text=Help('Select how BGP elements external to Checkmk will be created'),
                )),
            PARAM_MAKE_DEFAULT: DictElement(
                parameter_form=FixedValue(
                    title=Title('Make default'),
                    label=Label('This will be the default topology'),
                    help_text=Help(
                        'Makes the topology the default topology. If there no default topology, this '
                        'topology becomes always the default.'
                    ),
                    value=True
                )),
            PARAM_BGP_SITES_FILTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Filter Checkmk sites'),
                    elements=[
                        CascadingSingleChoiceElement(
                            name=PARAM_BGP_SITES_INCLUDE,
                            title=Title('Include checkmk sites'),
                            parameter_form=List(element_template=String(
                                title=Title('checkmk site name'),
                                custom_validate=(DuplicateInList(),),
                            ))
                        ),
                        CascadingSingleChoiceElement(
                            name=PARAM_BGP_SITES_EXCLUDE,
                            title=Title('Exclude checkmk sites'),
                            parameter_form=List(element_template=String(
                                title=Title('Checkmk site name'),
                                custom_validate=(DuplicateInList(),),
                            ))
                        )
                    ],
                    prefill=DefaultValue(PARAM_BGP_SITES_INCLUDE),
                    help_text=Help(''),
                )),
        }
    )


rule_spec_bgp_topo = ActiveCheck(
    name=RULE_SET_NAME_BGP_TOPOLOGY,
    topic=Topic.NETWORKING,
    parameter_form=_parameter_form,
    title=Title('BGP Topology'),
)
