#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-07-20
# File  : bgp_topology/server_side_calls/bgp_topology.py


from collections.abc import Iterator, Mapping, Sequence
from pydantic import BaseModel
from typing import Literal

from cmk.utils import debug
from cmk.server_side_calls.v1 import (
    HostConfig,
    Secret,
    ActiveCheckCommand,
    ActiveCheckConfig,
)

from cmk_addons.plugins.bgp_topology.constants import (
    ARG_PARSER_ANCHOR,
    ARG_PARSER_ANCHOR_AS,
    ARG_PARSER_ANCHOR_ID,
    ARG_PARSER_EMBLEM_AS,
    ARG_PARSER_EMBLEM_ID,
    ARG_PARSER_HOST,
    ARG_PARSER_MAKE_DEFAULT,
    ARG_PARSER_NONE,
    ARG_PARSER_SITES_EXCLUDE,
    ARG_PARSER_SITES_INCLUDE,
    PARAM_BGP_EXT_ANCHOR_AS,
    PARAM_BGP_EXT_ANCHOR_BOTH,
    PARAM_BGP_EXT_ANCHOR_ID,
    PARAM_BGP_EXT_ANCHOR_NONE,
    PARAM_BGP_SITES_EXCLUDE,
    PARAM_BGP_SITES_INCLUDE,
    PARAM_EMBLEM_CUSTOM,
    PARAM_EMBLEM_DEFAULT,
    PARAM_EMBLEM_NO_EMBLEM,
    RULE_SET_NAME_BGP_TOPOLOGY,
)

EMBLEM = (
        tuple[Literal["default_emblem"], bool] |
        tuple[Literal['custom_emblem'], str] |
        tuple[Literal['no_emblem'], bool] |
        None
)

FILTER_SITES = (
    tuple[Literal['exclude_sites'], Sequence[str]] |
    tuple[Literal['include_sites'], Sequence[str]]
)


class BgpAnchorType:
    anchor_both: str = PARAM_BGP_EXT_ANCHOR_BOTH
    anchor_as: str = PARAM_BGP_EXT_ANCHOR_AS
    anchor_id: str = PARAM_BGP_EXT_ANCHOR_ID
    anchor_none: str = PARAM_BGP_EXT_ANCHOR_NONE


class Emblem:
    default: str = PARAM_EMBLEM_DEFAULT
    custom: str = PARAM_EMBLEM_CUSTOM
    no_emblem: str = PARAM_EMBLEM_NO_EMBLEM


class Params(BaseModel):
    bgp_as: EMBLEM = None
    bgp_id: EMBLEM = None
    bgp_ext_anchor: str | None = None
    make_default: bool | None = None
    bgp_filter_sites: FILTER_SITES | None = None


__params = {
    'bgp_as': ('custom_emblem', 'icon_cloud'),
    'bgp_id': ('no_emblem', True),
    'bgp_ext_anchor': 'bgp_ext_anchor_both',
    'make_default': True
}
__parsed = Params(
    bgp_as=('custom_emblem', 'icon_cloud'),
    bgp_id=('no_emblem', True),
    bgp_ext_anchor='bgp_ext_anchor_both',
    make_default=True
)


def _commands_bgp_topology_parser(params: Mapping[str, object]) -> Params:
    if debug.enabled():
        print(params)
    return Params.model_validate(params)


def commands_bgp_topology_arguments(
        params: Params, host_config: HostConfig
) -> Iterator[ActiveCheckCommand]:
    if debug.enabled():
        pass
        # print(host_config)

    args: list[str | Secret] = []
    args += [ARG_PARSER_HOST, host_config.name.replace(' ', '_')]
    if params.make_default is True:
        args.append(ARG_PARSER_MAKE_DEFAULT)

    if params.bgp_ext_anchor is not None:
        match params.bgp_ext_anchor:
            case BgpAnchorType.anchor_both:
                pass  # this is the default
            case BgpAnchorType.anchor_as:
                args += [ARG_PARSER_ANCHOR, ARG_PARSER_ANCHOR_AS]
            case BgpAnchorType.anchor_id:
                args += [ARG_PARSER_ANCHOR, ARG_PARSER_ANCHOR_ID]
            case BgpAnchorType.anchor_none:
                args += [ARG_PARSER_ANCHOR, ARG_PARSER_NONE]
            case _:
                pass

    if params.bgp_id:
        key, value = params.bgp_id
        match key:
            case Emblem.default:
                pass  # this is the default
            case Emblem.custom:
                args += [ARG_PARSER_EMBLEM_ID, value]
            case Emblem.no_emblem:
                args += [ARG_PARSER_EMBLEM_ID, ARG_PARSER_NONE]
            case _:
                pass

    if params.bgp_as:
        key, value = params.bgp_as
        match key:
            case Emblem.default:
                pass  # this is the default
            case Emblem.custom:
                args += [ARG_PARSER_EMBLEM_AS, value]
            case Emblem.no_emblem:
                args += [ARG_PARSER_EMBLEM_AS, ARG_PARSER_NONE]
            case _:
                pass

    if params.bgp_filter_sites:
        class FilterMode:
            exclude_sites = PARAM_BGP_SITES_EXCLUDE
            include_sites = PARAM_BGP_SITES_INCLUDE

        mode, site_list = params.bgp_filter_sites
        match mode:
            case FilterMode.exclude_sites:
                args.append(ARG_PARSER_SITES_EXCLUDE)
                args += site_list
            case FilterMode.include_sites:
                args.append(ARG_PARSER_SITES_INCLUDE)
                args += site_list
            case _:
                pass

    if debug.enabled():
        print(args)

    yield ActiveCheckCommand(
        service_description="BGP Topology",
        command_arguments=args
    )


active_check_bgp_topology = ActiveCheckConfig(
    name=RULE_SET_NAME_BGP_TOPOLOGY,
    parameter_parser=_commands_bgp_topology_parser,
    commands_function=commands_bgp_topology_arguments,
)
